/*
<<<<<<< HEAD
    Name:       Happy16 Light
=======
    Name:       grade3
>>>>>>> 37f3f3b3505748599859a693d2868ea7e034059f
    Author:     Kyle Dunovan (http://github.com/dunovank)
*/
<style>
    body {background-color: #ffffff;}

    a {color: #303030; font-family: 'Helvetica', sans-serif; }
    div#notebook, div.CodeMirror, div.output_area pre, div.output_wrapper, div.prompt, div.CodeMirror pre {font-family:'Hack', monospace; font-size: 11pt;}

    /* Hiding the ipython log */
    div#ipython_notebook { display: none; }

    /* INTRO PAGE */
    .toolbar_info { color: #2c2c2c; font-size: 12pt; border: 0; font-family: 'Helvetica', sans-serif; text-shadow: none !important; }
    .list_container {color: #2c2c2c; font-size: 12pt; border: 0; font-family: 'Helvetica', sans-serif;text-shadow: none !important; }
    .breadcrumb>li {text-shadow:0 0px 0; color:#2c2c2c; font-family: 'Helvetica', sans-serif; font-size: 12pt}
    .list_container a {color: #2c2c2c; font-family: 'Helvetica', sans-serif; font-size: 12pt; border: 0; text-shadow: none !important; }
    .item_icon {color: #3572C6;}
    .item_buttons .kernel-name {
        color: #3572C6;
    }
    .running_notebook_icon:before {
        color: #ff8132 !important;
    }
    .item_buttons .running-indicator {
        padding-top: 4px;
        color: #ff8132;
    }
    .nav-tabs > li > a:hover {
        color: #222222;
        background: #ffffff;
        background-color: #ffffff;
        background-image: none;
        border: 1px solid #ff8132;
        border-bottom-color: #ffffff;
    }
    .nav-tabs > li.active > a, .nav-tabs > li.active > a:hover, .nav-tabs > li.active > a:focus {
        color: #222222;
        background: #ffffff;
        background-color: #ffffff;
        border: 1px solid #ff8132;
        border-bottom-color: #ffffff;
    }
    .nav-tabs {
        border-bottom: 1px solid #ff8132;
    }

    /*<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>
    *****************************************
    COMMENT THIS LINE OUT TO RESTORE TOOLBAR
    *****************************************
    <<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>> */
    div#maintoolbar {display: none !important;}
    .list_header {
        font-weight: normal;
        background-color: transparent;
        color: #4c4c4c;
    }
    div.list_container a:hover {color: #222222;}
    div.list_item:hover {
        background-color: rgba(184, 184, 184, 0.10);
    }
    ul#selector-menu.dropdown-menu > li > a {
        background: #efefef;
        background-color: #efefef;
    }
    ul#selector-menu.dropdown-menu > li > a:hover {
        background: #efefef;
<<<<<<< HEAD
        background-color: rgba(184, 184, 184, 0.30);
=======
        background-color: rgba(184, 184, 184, 0.10);
>>>>>>> 37f3f3b3505748599859a693d2868ea7e034059f
    }

    /* PANELS // LISTS // DROPDOWN MENUS*/
    .dropdown-menu {
        background-color: #efefef;
        background: #efefef;
        border: 2px solid #dddddd;
        box-shadow: none;
        padding: 0px 0;
        font-size: 12pt;
        text-align: left;
        vertical-align: middle;
    }
    .dropdown-menu:hover {
        background-color: #efefef;
        border: 2px solid #dddddd;
        box-shadow: none;
        vertical-align: middle;
    }
    .dropdown-header {
        font-size: 13pt;
        display: block;
        line-height: 1.25;
        color: #333333;
        text-decoration: underline;
        text-decoration-color: rgba(184, 184, 184, 0.40);
        white-space: nowrap;
    }
    .dropdown-menu > li > a {
        font-size: 12pt;
        display: block;
        clear: both;
        padding: 8px 20px;
        font-weight: normal;
        line-height: 1.25;
        color: #333333;
        white-space: nowrap;
        background-color: #efefef;
        background: #efefef;
    }
    .dropdown-menu > li > a:hover {
        font-size: 12pt;
        color: #111111;
        display: block;
        clear: both;
        padding: 8px 20px;
        font-weight: normal;
        line-height: 1.25;
        white-space: nowrap;
        background: #efefef;
        background-color: rgba(184, 184, 184, 0.30);
    }
    .dropdown-menu .divider {
        height: 2px;
        margin: 0px 0px;
        overflow: hidden;
        background-color: #dddddd;
    }
    .dropdown-submenu > .dropdown-menu {
        top: 0;
        left: 100%;
        margin-top: -2px;
        margin-left: 0px;
        padding-top: 0px;
    }
    .dropdown-menu > .disabled > a, .dropdown-menu > .disabled > a:hover, .dropdown-menu > .disabled > a:focus {
        color: #aaaaaa;
    }
    .modal-header {
        padding: 15px;
        border-bottom: 1px solid #ffffff;
        min-height: 16.4286px;
    }
    .modal-content {
        position: relative;
        background-color: #ffffff;
        border: 1px solid #808080;
        border-radius: 3px;
        box-shadow: 0px 3px 9px #808080;
        background-clip: padding-box;
        outline: 0px none;
    }
    .panel {
        line-height: 22px;
        fontsize: 12px;
        margin-bottom: 18px;
        background-color: #ffffff;
        color: #2c2c2c;
        box-shadow: none;
    }
    #running .panel-group .panel .panel-heading {
        background-color: #ffffff;
        padding: 4px 7px;
        line-height: 22px;
    }
    #running .panel-group .panel .panel-heading a:hover {
        color: #ff8132;
    }
    #running .panel-group .panel .panel-body .list_container .list_item {
        border-bottom: 1px solid rgba(224, 224, 224, 0.55);
    }
    #project_name > .breadcrumb {
        font-weight: normal;
        color: #ff8132;
    }
    .rendered_html pre, .rendered_html code {
        font-size: 12pt;
        color: #2c2c2c;
        line-height: 100%;
        background: #dfdfdf;
    }
    body {
        font-family: "Helvetica", Helvetica, Arial, sans-serif;
        font-size: 12pt;
        line-height: 1.42857;
        color: #2c2c2c;
    }
    #kernel_indicator {
        float: right !important;
        color: #ff8132;
    }
    #kernel_indicator .kernel_indicator_name {
        padding-left: 5px;
        padding-right: 5px;
        font-size: 13pt;
    }
    #modal_indicator {
        float: right !important;
        color: #ff8132;

    }
    .notebook_app {
        background-color: #ffffff;
    }
    .notebook_app #header {
        box-shadow: none !important;
        background-color: #ffffff;
    }
    #notebook-container {
        padding: 15px;
        background-color: #ffffff;
        min-height: 0px;
        box-shadow: none;
    }
    #header {
        box-shadow: none;
        background-color: #ffffff;
        position: relative;
        z-index: 100;
    }
    #header-container {
        padding-bottom: 5px;
        display: none !important;
        box-shadow: none;
    }
    #header .header-bar {
        background: #ffffff;
    }
    /* text in output area (i.e. print some_list) */
    div.output_html {color: #1e2429;}

    /* Formatting code in text, code*/
    .rendered_html table, .rendered_html th, .rendered_html tr, .rendered_html td {
      border: 1px #4c4c4c; solid;
      color: #4c4c4c;
      background-color: #ffffff !important;
      align-content: left !important;
    }
    .navbar-default {
        background-color: #ffffff;
        border-color: #ffffff;
    }
    #menubar .navbar {
        border-top: 1px;
        border-radius: 0 0 2px 2px;
        margin-bottom: 0;
        margin-top: 8px;
    }
    .navbar-default .navbar-nav > li > a {
        color: #333333;
        font-size: 14pt;
    }
    .navbar-default .navbar-nav > li > a:hover, .navbar-default .navbar-nav > li > a:focus {
        color: #222222;
        background-color: #efefef;
    }
    .navbar-default .navbar-nav > .open > a, .navbar-default .navbar-nav > .open > a:hover, .navbar-default .navbar-nav > .open > a:focus {
        color: #222222;
        background-color: #efefef;
    }
    .btn {
        color: #4c4c4c;
        background: transparent;
        background-color: transparent;
        font-size: 12pt;
        font-weight: normal;
        border: 0 none !important;
        box-shadow: none !important;
        text-shadow: none;
        vertical-align: 0px;
    }
    .btn-group > .btn-mini {
        color: #4c4c4c;
        background: transparent;
        background-color: transparent;
        font-size: 11pt;
        border-radius: 2px;
        font-weight: normal;
        border: 0 none !important;
        box-shadow: none !important;
        text-shadow: none;
    }
    .btn:hover {
        color: #222222;
        background: transparent;
<<<<<<< HEAD
        background-color: #dddddd;
=======
        background-color: #efefef;
>>>>>>> 37f3f3b3505748599859a693d2868ea7e034059f
        background-image: none;
    }
    .btn-group > .btn-mini:hover {
        color: #222222;
        background: transparent;
<<<<<<< HEAD
        background-color: #dddddd;
=======
        background-color: #efefef;
>>>>>>> 37f3f3b3505748599859a693d2868ea7e034059f
        background-image: none;
    }
    .btn-group > .btn-group-vertical {
        background: transparent;
        background-color: transparent;
        position: relative;
        display: inline-block;
        vertical-align: middle;
    }
    .btn-group > .btn-group-vertical:hover {
        color: #222222;
        background: transparent;
        background-color: transparent;
        background-image: none;
        border-radius: 2px;
    }
    .btn-group > .btn-default.btn-xs:hover {
        color: #222222;
        background: transparent;
<<<<<<< HEAD
        background-color: #dddddd;
=======
        background-color: #efefef;
>>>>>>> 37f3f3b3505748599859a693d2868ea7e034059f
        background-image: none;
    }
    .alternate_upload .btn-upload {
        padding: 1px 5px;
        line-height: 1.5;
        height: 26px;
        background: transparent;
        display: inline-block;
    }
    .alternate_upload .btn-upload:hover {
        color: #222222;
        background: transparent;
<<<<<<< HEAD
        background-color: #dddddd;
=======
        background-color: #efefef;
>>>>>>> 37f3f3b3505748599859a693d2868ea7e034059f
        background-image: none;
    }
    .btn-default.active:hover, .open > .dropdown-toggle.btn-default:hover, .btn-default:active:focus, .btn-default.active:focus, .open > .dropdown-toggle.btn-default:focus, .btn-default:active.focus, .btn-default.active.focus, .open > .dropdown-toggle.btn-default.focus {
        color: #222222;
        background: transparent;
        background-color: #efefef;
    }

    .ipython_tooltip {
        border: 1px solid #aaaaaa;
        box-shadow: none;
        border-radius: 2px;
        position: absolute;
        z-index: 1000;
        background-color: #dddddd;
        color: #222222;
    }
    .ipython_tooltip .tooltiptext pre {
        border: 0px none;
        border-radius: 0px;
        background-color: #dddddd;
        color: #222222;
    }
    /* TOOLTIP DIMENSIONS */
    div#tooltip.ipython_tooltip {
        overflow: auto;
        max-width: 900px;
    }
    div.tooltiptext.bigtooltip {
        overflow: auto;
        height: 400px;
        width: 900px;
    }
    /* Formatting code in markdown cells */
    div.text_cell_render {
        font-family: 'Helvetica', sans-serif;
        background: #ededed;
        font-weight: 310;
        color: #4c4c4c;
    }
    /* Formatting code in markdown cells */
    div.text_cell_render pre {
        color: #4c4c4c;
        background-color: #ededed;
    }
    div.text_cell {
        max-width: 100%;
        color: #4c4c4c;
    }
    div.text_cell_render h1,
    div.text_cell_render h2,
    div.text_cell_render h3,
    div.text_cell_render h4,
    div.text_cell_render h5,
    div.text_cell_render h6 {
        font-family: 'Helvetica', sans-serif;
<<<<<<< HEAD
        background: #ececed;
=======
        background: #ededed;
>>>>>>> 37f3f3b3505748599859a693d2868ea7e034059f
        font-weight: 300;
        color: #4c4c4c;
        line-height: 90%;
    }

    /* text in output area (i.e. print some_list) */
    div.output_html {color: #4c4c4c;}

    /* Formatting code in text, code*/
    .rendered_html table, .rendered_html th, .rendered_html tr, .rendered_html td {
      border: 1px #4c4c4c solid;
      color: #4c4c4c;
<<<<<<< HEAD
      background-color: #ececed !important;
=======
      background-color: #ededed !important;
>>>>>>> 37f3f3b3505748599859a693d2868ea7e034059f
      align-content: left !important;
    }

    button.close {
        padding: 0px;
        cursor: pointer;
        background: transparent none repeat scroll 0% 0%;
        border: 0px none;
    }
    .close {
        float: right;
        font-size: 22px;
        font-weight: bold;
        line-height: 1;
        color: #ff8132;
        text-shadow: 0px 1px 0px rgba(240, 147, 43, .2);
        opacity: 0.3;
    }
    .close:hover {
        color: #ff8132;
        text-shadow: 0px 1px 0px rgba(240, 147, 43, .2);
        opacity: 0.9;
    }
    .form-control {
        display: block;
        width: 100%;
        height: 32px;
        padding: 6px 12px;
        font-size: 13pt;
        line-height: 1.6;
        color: #4c4c4c;
<<<<<<< HEAD
        background-color: #ececed;
=======
        background-color: #ededed;
>>>>>>> 37f3f3b3505748599859a693d2868ea7e034059f
        background-image: none;
        border: 1px solid rgba(240, 147, 43, .8);
        border-radius: 2px;
        box-shadow: none;
        transition: border-color 0.15s ease-in-out 0s, box-shadow 0.15s ease-in-out 0s;
    }
    .list_item input:not([type="checkbox"]) {
        margin-right: 0px;
        padding-right: 0px;
        height: 80%;
        width: 30%;
    }
    .modal-footer {
        padding: 15px;
        text-align: right;
        border-top: 1px solid rgba(184, 184, 184, 0.75);
    }
    .input-form-control:hover:active {
        background-color: rgba(184, 184, 184, 0.75);
        selected: rgba(184, 184, 184, 0.75);
    }

    span.ansiblack {color: #2b303b;}
    span.ansiblue {color: #154ba1;}
    span.ansigray {color: #b4b7b4;}
    span.ansigreen {color: #2d6836;}
    span.ansipurple {color: #483d8b;}
    span.ansicyan {color: #1592a2;}
    span.ansiyellow {color: #ee8713;}
    span.ansired {color: #b82a18;}

    div.output_stderr {background-color: #DB797C;}
    div.output_stderr pre {color: #000000;}

    .cm-s-ipython.CodeMirror {background: #ededed; color: #303030;}
    .cm-s-ipython div.CodeMirror-selected {background: #D3D4D7 !important;}
    .cm-s-ipython .CodeMirror-gutters {background: #cccccc; border: 0px; border-radius:0px;}
    .cm-s-ipython .CodeMirror-linenumber {color: #666666 !important; font-size: 11pt;}
    .cm-s-ipython .CodeMirror-cursor {border-left: 2px solid #0095ff !important;}
    .cm-s-ipython span.cm-comment {color: #707070; font-style: italic !important;}
    .cm-s-ipython span.cm-atom {color: #055be0;}
    .cm-s-ipython span.cm-number {color: #ff8132;}
    .cm-s-ipython span.cm-property {color: #de143d;}
    .cm-s-ipython span.cm-attribute {color: #de143d;}
    .cm-s-ipython span.cm-keyword {color: #653bc5; font-weight: normal;}
    .cm-s-ipython span.cm-string {color: #069c00; }
    .cm-s-ipython span.cm-operator {color: #055be0; }
    .cm-s-ipython span.cm-builtin {color: #de143d; }
    .cm-s-ipython span.cm-variable {color: #303030;}

    .cm-s-ipython span.cm-def {color: #e22978; font-weight: normal;}
    .cm-s-ipython span.cm-error {background: rgba(191, 97, 106, .3) !important;}
    .cm-s-ipython span.cm-tag {color: #e22978;}
    .cm-s-ipython span.cm-link {color: #ff8132;}
    .cm-s-ipython span.cm-storage {color: #055be0;}
    .cm-s-ipython span.cm-entity {color: #de143d;}
    .cm-s-ipython span.cm-class {color: #e22978;}
    .cm-s-ipython span.cm-support {color: #de143d;}
    .cm-s-ipython span.cm-qualifier {color: #de143d;}

    /* Set CodeCell font-size*/
    /*div.CodeMirror pre {font-size: 11pt !important;}*/
<<<<<<< HEAD
    div.CodeMirror pre {line-height: 1.5 !important;  background-color: #ededed; }
=======
    div.CodeMirror pre {line-height: 1.5 !important;}
>>>>>>> 37f3f3b3505748599859a693d2868ea7e034059f
    div.CodeMirror pre {font-family: 'Hack', monospace; font-size: 11pt;}

    div.CodeMirror span.CodeMirror-matchingbracket { color: #000000; background: rgba(53, 114, 198, .4) !important ; height: 120%; line-height: 1.5 !important; padding-top: 1px;}
    div.CodeMirror span.CodeMirror-nonmatchingbracket { color: #000000; background: rgba(191, 97, 106, .3) !important ; height: 120%; line-height: 1.5 !important; padding-top: 1px;}

<<<<<<< HEAD
    div.cell { background-color: #ededed; border: medium solid #ffffff; padding: 0px; border-radius: 0px; }
    div.cell.selected { background-color: #ededed; border: medium solid #ff8132; padding: 0px; border-radius: 0px; }
    .edit_mode div.cell.selected {border: medium solid #ffd5bb !important; padding: 0px; border-radius: 0px; }
    div.cell.edit_mode { background-color: #ededed; border: medium solid #ffd5bb !important; padding: 0px; border-radius: 0px;}

=======
    div.cell { border: medium solid #ffffff; padding: 0px; border-radius: 0px; }
    div.cell.selected { border: medium solid #ff8132; padding: 0px; border-radius: 0px; }
    .edit_mode div.cell.selected {border: medium solid #ffd5bb !important; padding: 0px; border-radius: 0px; }
    div.cell.edit_mode {border: medium solid #ffd5bb !important; padding: 0px; border-radius: 0px;}
    div.cell_render {background-color: #ededed}
    div.cell {background-color: #ededed}
>>>>>>> 37f3f3b3505748599859a693d2868ea7e034059f
    div.prompt {
        padding: 0.3em;
        margin: 0px;
        font-family: monospace;
        text-align: left;
        line-height: 1.21429em;
        min-width: 11ex;
        font-size: 9.5pt;
    }
<<<<<<< HEAD
    div.input_area {
        background-color: #ededed;
=======

    div.input_area {
        background-color: #ededed;
        background: #ededed;
>>>>>>> 37f3f3b3505748599859a693d2868ea7e034059f
        margin-bottom: 0px;
        border: 0px;
        border-radius: 0px;
    }
    /*Input and output cells formatting*/
    div.prompt.input_prompt {
        font-family: 'Hack', monospace;
        border-right: 2px solid rgba(240, 147, 43, .4);
        font-weight: normal;
        font-size: 9.5pt;
        color: #aaaaaa;
        background-color: #ededed;
        width: 1em;
        padding-bottom: 0px;
        margin-bottom: 0px;
    }
<<<<<<< HEAD
=======

>>>>>>> 37f3f3b3505748599859a693d2868ea7e034059f
    div.prompt.output_prompt {
        font-family: 'Hack', monospace;
        font-weight: bold;
        background-color: #ffffff;
        color: #ffffff;
        font-size: 9.5pt;
        width: 1em;
        margin-bottom: 8px;
        padding-top: 1em;
        padding-bottom: 1em;
    }
    div.out_prompt_overlay.prompt {
        font-family: 'Hack', monospace;
        font-weight: bold;
        font-size: 9.5pt;
        width: 1em;
        margin-bottom: 8px;
        background-color: #ffffff;
        border-radius: 4px;
        -webkit-border-radius: 4px;
        -moz-border-radius: 4px;
    }
    div.out_prompt_overlay.prompt:hover {
        font-family: 'Hack', monospace;
        font-weight: bold;
        font-size: 9.5pt;
        width: 1em;
        border-radius: 4px;
        margin-bottom: 8px;
        background-color: #ffffff;
        box-shadow: #e2e2e2 2px 2px 6px 4px inset;
        -webkit-border-radius: 4px;
        -moz-border-radius: 4px;
    }

    div.output_wrapper {
        background-color:#ffffff;
        margin-bottom: 0em;
        margin-top: 0em;
        border: 0px;
    }
    div.output_subarea.output_text.output_stream.output_stdout,div.output_subarea.output_text {
        padding-top: 1em;
        margin-bottom: 8px;
        margin-top: 8px;
        margin-left: 8px;
        margin-right: 8px;
        background-color: #ffffff;
        color: #000000;
    }

    div.output_area pre {
        color: #000000;
        font-size: 10pt;
    }
    div.output_subarea {
        overflow-x: auto;
        padding: 1.0em;
        -webkit-box-flex: 1;
        -moz-box-flex: 1;
        box-flex: 1;
        flex: 1;
        max-width: calc(100% - 14ex);
    }
<<<<<<< HEAD
=======

>>>>>>> 37f3f3b3505748599859a693d2868ea7e034059f
</style>
